# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mtap/api/v1/events.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18mtap/api/v1/events.proto\x12\x0bmtap.api.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/protobuf/struct.proto\"\x1c\n\x1aGetEventsInstanceIdRequest\"2\n\x1bGetEventsInstanceIdResponse\x12\x13\n\x0binstance_id\x18\x01 \x01(\t\"p\n\x10OpenEventRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x17\n\x0fonly_create_new\x18\x02 \x01(\x08\x12\x31\n\x0elease_duration\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\"6\n\x11OpenEventResponse\x12\x0f\n\x07\x63reated\x18\x01 \x01(\x08\x12\x10\n\x08lease_id\x18\x02 \x01(\x05\"7\n\x11\x43loseEventRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x10\n\x08lease_id\x18\x02 \x01(\x05\"%\n\x12\x43loseEventResponse\x12\x0f\n\x07\x64\x65leted\x18\x01 \x01(\x08\")\n\x15GetAllMetadataRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\"\x8e\x01\n\x16GetAllMetadataResponse\x12\x43\n\x08metadata\x18\x01 \x03(\x0b\x32\x31.mtap.api.v1.GetAllMetadataResponse.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"B\n\x12\x41\x64\x64MetadataRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\"\x15\n\x13\x41\x64\x64MetadataResponse\"0\n\x1cGetAllBinaryDataNamesRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\":\n\x1dGetAllBinaryDataNamesResponse\x12\x19\n\x11\x62inary_data_names\x18\x01 \x03(\t\"W\n\x14\x41\x64\x64\x42inaryDataRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x18\n\x10\x62inary_data_name\x18\x02 \x01(\t\x12\x13\n\x0b\x62inary_data\x18\x03 \x01(\x0c\"\x17\n\x15\x41\x64\x64\x42inaryDataResponse\"B\n\x14GetBinaryDataRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x18\n\x10\x62inary_data_name\x18\x02 \x01(\t\",\n\x15GetBinaryDataResponse\x12\x13\n\x0b\x62inary_data\x18\x01 \x01(\x0c\"K\n\x12\x41\x64\x64\x44ocumentRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\x12\x0c\n\x04text\x18\x03 \x01(\t\"\x15\n\x13\x41\x64\x64\x44ocumentResponse\".\n\x1aGetAllDocumentNamesRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\"5\n\x1bGetAllDocumentNamesResponse\x12\x16\n\x0e\x64ocument_names\x18\x01 \x03(\t\"A\n\x16GetDocumentTextRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\"\'\n\x17GetDocumentTextResponse\x12\x0c\n\x04text\x18\x01 \x01(\t\"E\n\x1aGetLabelIndicesInfoRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\"\xa6\x02\n\x1bGetLabelIndicesInfoResponse\x12R\n\x11label_index_infos\x18\x01 \x03(\x0b\x32\x37.mtap.api.v1.GetLabelIndicesInfoResponse.LabelIndexInfo\x1a\xb2\x01\n\x0eLabelIndexInfo\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12T\n\x04type\x18\x02 \x01(\x0e\x32\x46.mtap.api.v1.GetLabelIndicesInfoResponse.LabelIndexInfo.LabelIndexType\"6\n\x0eLabelIndexType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07GENERIC\x10\x01\x12\n\n\x06\x43USTOM\x10\x02\"\xd9\x01\n\x10\x41\x64\x64LabelsRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\x12\x12\n\nindex_name\x18\x03 \x01(\t\x12\x34\n\x0egeneric_labels\x18\x04 \x01(\x0b\x32\x1a.mtap.api.v1.GenericLabelsH\x00\x12-\n\rcustom_labels\x18\x05 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x12\x19\n\x11no_key_validation\x18\x06 \x01(\x08\x42\x08\n\x06labels\"\x13\n\x11\x41\x64\x64LabelsResponse\"O\n\x10GetLabelsRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\x12\x12\n\nindex_name\x18\x03 \x01(\t\"O\n\rGenericLabels\x12\x13\n\x0bis_distinct\x18\x01 \x01(\x08\x12)\n\x06labels\x18\x02 \x03(\x0b\x32\x19.mtap.api.v1.GenericLabel\"\xa3\x01\n\x0cGenericLabel\x12\x12\n\nidentifier\x18\x01 \x01(\r\x12\x13\n\x0bstart_index\x18\x02 \x01(\r\x12\x11\n\tend_index\x18\x03 \x01(\r\x12\'\n\x06\x66ields\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12.\n\rreference_ids\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\"\x82\x01\n\x11GetLabelsResponse\x12\x34\n\x0egeneric_labels\x18\x01 \x01(\x0b\x32\x1a.mtap.api.v1.GenericLabelsH\x00\x12-\n\rcustom_labels\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x42\x08\n\x06labels2\xd8\x0f\n\x06\x45vents\x12\x88\x01\n\x13GetEventsInstanceId\x12\'.mtap.api.v1.GetEventsInstanceIdRequest\x1a(.mtap.api.v1.GetEventsInstanceIdResponse\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16/v1/events/instance_id\x12l\n\tOpenEvent\x12\x1d.mtap.api.v1.OpenEventRequest\x1a\x1e.mtap.api.v1.OpenEventResponse\" \x82\xd3\xe4\x93\x02\x1a\"\x15/v1/events/{event_id}:\x01*\x12l\n\nCloseEvent\x12\x1e.mtap.api.v1.CloseEventRequest\x1a\x1f.mtap.api.v1.CloseEventResponse\"\x1d\x82\xd3\xe4\x93\x02\x17*\x15/v1/events/{event_id}\x12\x81\x01\n\x0eGetAllMetadata\x12\".mtap.api.v1.GetAllMetadataRequest\x1a#.mtap.api.v1.GetAllMetadataResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/v1/events/{event_id}/metadata\x12\x81\x01\n\x0b\x41\x64\x64Metadata\x12\x1f.mtap.api.v1.AddMetadataRequest\x1a .mtap.api.v1.AddMetadataResponse\"/\x82\xd3\xe4\x93\x02)\"$/v1/events/{event_id}/metadata/{key}:\x01*\x12\x8c\x01\n\x0b\x41\x64\x64\x44ocument\x12\x1f.mtap.api.v1.AddDocumentRequest\x1a .mtap.api.v1.AddDocumentResponse\":\x82\xd3\xe4\x93\x02\x34\"//v1/events/{event_id}/documents/{document_name}:\x01*\x12\x91\x01\n\x13GetAllDocumentNames\x12\'.mtap.api.v1.GetAllDocumentNamesRequest\x1a(.mtap.api.v1.GetAllDocumentNamesResponse\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/v1/events/{event_id}/documents\x12\x95\x01\n\x0fGetDocumentText\x12#.mtap.api.v1.GetDocumentTextRequest\x1a$.mtap.api.v1.GetDocumentTextResponse\"7\x82\xd3\xe4\x93\x02\x31\x12//v1/events/{event_id}/documents/{document_name}\x12\xa8\x01\n\x13GetLabelIndicesInfo\x12\'.mtap.api.v1.GetLabelIndicesInfoRequest\x1a(.mtap.api.v1.GetLabelIndicesInfoResponse\">\x82\xd3\xe4\x93\x02\x38\x12\x36/v1/events/{event_id}/documents/{document_name}/labels\x12\x9a\x01\n\tAddLabels\x12\x1d.mtap.api.v1.AddLabelsRequest\x1a\x1e.mtap.api.v1.AddLabelsResponse\"N\x82\xd3\xe4\x93\x02H\"C/v1/events/{event_id}/documents/{document_name}/labels/{index_name}:\x01*\x12\x97\x01\n\tGetLabels\x12\x1d.mtap.api.v1.GetLabelsRequest\x1a\x1e.mtap.api.v1.GetLabelsResponse\"K\x82\xd3\xe4\x93\x02\x45\x12\x43/v1/events/{event_id}/documents/{document_name}/labels/{index_name}\x12\x96\x01\n\x15GetAllBinaryDataNames\x12).mtap.api.v1.GetAllBinaryDataNamesRequest\x1a*.mtap.api.v1.GetAllBinaryDataNamesResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/v1/events/{event_id}/binaries\x12\x94\x01\n\rAddBinaryData\x12!.mtap.api.v1.AddBinaryDataRequest\x1a\".mtap.api.v1.AddBinaryDataResponse\"<\x82\xd3\xe4\x93\x02\x36\"1/v1/events/{event_id}/binaries/{binary_data_name}:\x01*\x12\x91\x01\n\rGetBinaryData\x12!.mtap.api.v1.GetBinaryDataRequest\x1a\".mtap.api.v1.GetBinaryDataResponse\"9\x82\xd3\xe4\x93\x02\x33\x12\x31/v1/events/{event_id}/binaries/{binary_data_name}B\x1b\n\x19\x65\x64u.umn.nlpie.mtap.api.v1b\x06proto3')



_GETEVENTSINSTANCEIDREQUEST = DESCRIPTOR.message_types_by_name['GetEventsInstanceIdRequest']
_GETEVENTSINSTANCEIDRESPONSE = DESCRIPTOR.message_types_by_name['GetEventsInstanceIdResponse']
_OPENEVENTREQUEST = DESCRIPTOR.message_types_by_name['OpenEventRequest']
_OPENEVENTRESPONSE = DESCRIPTOR.message_types_by_name['OpenEventResponse']
_CLOSEEVENTREQUEST = DESCRIPTOR.message_types_by_name['CloseEventRequest']
_CLOSEEVENTRESPONSE = DESCRIPTOR.message_types_by_name['CloseEventResponse']
_GETALLMETADATAREQUEST = DESCRIPTOR.message_types_by_name['GetAllMetadataRequest']
_GETALLMETADATARESPONSE = DESCRIPTOR.message_types_by_name['GetAllMetadataResponse']
_GETALLMETADATARESPONSE_METADATAENTRY = _GETALLMETADATARESPONSE.nested_types_by_name['MetadataEntry']
_ADDMETADATAREQUEST = DESCRIPTOR.message_types_by_name['AddMetadataRequest']
_ADDMETADATARESPONSE = DESCRIPTOR.message_types_by_name['AddMetadataResponse']
_GETALLBINARYDATANAMESREQUEST = DESCRIPTOR.message_types_by_name['GetAllBinaryDataNamesRequest']
_GETALLBINARYDATANAMESRESPONSE = DESCRIPTOR.message_types_by_name['GetAllBinaryDataNamesResponse']
_ADDBINARYDATAREQUEST = DESCRIPTOR.message_types_by_name['AddBinaryDataRequest']
_ADDBINARYDATARESPONSE = DESCRIPTOR.message_types_by_name['AddBinaryDataResponse']
_GETBINARYDATAREQUEST = DESCRIPTOR.message_types_by_name['GetBinaryDataRequest']
_GETBINARYDATARESPONSE = DESCRIPTOR.message_types_by_name['GetBinaryDataResponse']
_ADDDOCUMENTREQUEST = DESCRIPTOR.message_types_by_name['AddDocumentRequest']
_ADDDOCUMENTRESPONSE = DESCRIPTOR.message_types_by_name['AddDocumentResponse']
_GETALLDOCUMENTNAMESREQUEST = DESCRIPTOR.message_types_by_name['GetAllDocumentNamesRequest']
_GETALLDOCUMENTNAMESRESPONSE = DESCRIPTOR.message_types_by_name['GetAllDocumentNamesResponse']
_GETDOCUMENTTEXTREQUEST = DESCRIPTOR.message_types_by_name['GetDocumentTextRequest']
_GETDOCUMENTTEXTRESPONSE = DESCRIPTOR.message_types_by_name['GetDocumentTextResponse']
_GETLABELINDICESINFOREQUEST = DESCRIPTOR.message_types_by_name['GetLabelIndicesInfoRequest']
_GETLABELINDICESINFORESPONSE = DESCRIPTOR.message_types_by_name['GetLabelIndicesInfoResponse']
_GETLABELINDICESINFORESPONSE_LABELINDEXINFO = _GETLABELINDICESINFORESPONSE.nested_types_by_name['LabelIndexInfo']
_ADDLABELSREQUEST = DESCRIPTOR.message_types_by_name['AddLabelsRequest']
_ADDLABELSRESPONSE = DESCRIPTOR.message_types_by_name['AddLabelsResponse']
_GETLABELSREQUEST = DESCRIPTOR.message_types_by_name['GetLabelsRequest']
_GENERICLABELS = DESCRIPTOR.message_types_by_name['GenericLabels']
_GENERICLABEL = DESCRIPTOR.message_types_by_name['GenericLabel']
_GETLABELSRESPONSE = DESCRIPTOR.message_types_by_name['GetLabelsResponse']
_GETLABELINDICESINFORESPONSE_LABELINDEXINFO_LABELINDEXTYPE = _GETLABELINDICESINFORESPONSE_LABELINDEXINFO.enum_types_by_name['LabelIndexType']
GetEventsInstanceIdRequest = _reflection.GeneratedProtocolMessageType('GetEventsInstanceIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETEVENTSINSTANCEIDREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetEventsInstanceIdRequest)
  })
_sym_db.RegisterMessage(GetEventsInstanceIdRequest)

GetEventsInstanceIdResponse = _reflection.GeneratedProtocolMessageType('GetEventsInstanceIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETEVENTSINSTANCEIDRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetEventsInstanceIdResponse)
  })
_sym_db.RegisterMessage(GetEventsInstanceIdResponse)

OpenEventRequest = _reflection.GeneratedProtocolMessageType('OpenEventRequest', (_message.Message,), {
  'DESCRIPTOR' : _OPENEVENTREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.OpenEventRequest)
  })
_sym_db.RegisterMessage(OpenEventRequest)

OpenEventResponse = _reflection.GeneratedProtocolMessageType('OpenEventResponse', (_message.Message,), {
  'DESCRIPTOR' : _OPENEVENTRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.OpenEventResponse)
  })
_sym_db.RegisterMessage(OpenEventResponse)

CloseEventRequest = _reflection.GeneratedProtocolMessageType('CloseEventRequest', (_message.Message,), {
  'DESCRIPTOR' : _CLOSEEVENTREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.CloseEventRequest)
  })
_sym_db.RegisterMessage(CloseEventRequest)

CloseEventResponse = _reflection.GeneratedProtocolMessageType('CloseEventResponse', (_message.Message,), {
  'DESCRIPTOR' : _CLOSEEVENTRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.CloseEventResponse)
  })
_sym_db.RegisterMessage(CloseEventResponse)

GetAllMetadataRequest = _reflection.GeneratedProtocolMessageType('GetAllMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETALLMETADATAREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllMetadataRequest)
  })
_sym_db.RegisterMessage(GetAllMetadataRequest)

GetAllMetadataResponse = _reflection.GeneratedProtocolMessageType('GetAllMetadataResponse', (_message.Message,), {

  'MetadataEntry' : _reflection.GeneratedProtocolMessageType('MetadataEntry', (_message.Message,), {
    'DESCRIPTOR' : _GETALLMETADATARESPONSE_METADATAENTRY,
    '__module__' : 'mtap.api.v1.events_pb2'
    # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllMetadataResponse.MetadataEntry)
    })
  ,
  'DESCRIPTOR' : _GETALLMETADATARESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllMetadataResponse)
  })
_sym_db.RegisterMessage(GetAllMetadataResponse)
_sym_db.RegisterMessage(GetAllMetadataResponse.MetadataEntry)

AddMetadataRequest = _reflection.GeneratedProtocolMessageType('AddMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDMETADATAREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddMetadataRequest)
  })
_sym_db.RegisterMessage(AddMetadataRequest)

AddMetadataResponse = _reflection.GeneratedProtocolMessageType('AddMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDMETADATARESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddMetadataResponse)
  })
_sym_db.RegisterMessage(AddMetadataResponse)

GetAllBinaryDataNamesRequest = _reflection.GeneratedProtocolMessageType('GetAllBinaryDataNamesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETALLBINARYDATANAMESREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllBinaryDataNamesRequest)
  })
_sym_db.RegisterMessage(GetAllBinaryDataNamesRequest)

GetAllBinaryDataNamesResponse = _reflection.GeneratedProtocolMessageType('GetAllBinaryDataNamesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETALLBINARYDATANAMESRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllBinaryDataNamesResponse)
  })
_sym_db.RegisterMessage(GetAllBinaryDataNamesResponse)

AddBinaryDataRequest = _reflection.GeneratedProtocolMessageType('AddBinaryDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDBINARYDATAREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddBinaryDataRequest)
  })
_sym_db.RegisterMessage(AddBinaryDataRequest)

AddBinaryDataResponse = _reflection.GeneratedProtocolMessageType('AddBinaryDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDBINARYDATARESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddBinaryDataResponse)
  })
_sym_db.RegisterMessage(AddBinaryDataResponse)

GetBinaryDataRequest = _reflection.GeneratedProtocolMessageType('GetBinaryDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBINARYDATAREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetBinaryDataRequest)
  })
_sym_db.RegisterMessage(GetBinaryDataRequest)

GetBinaryDataResponse = _reflection.GeneratedProtocolMessageType('GetBinaryDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETBINARYDATARESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetBinaryDataResponse)
  })
_sym_db.RegisterMessage(GetBinaryDataResponse)

AddDocumentRequest = _reflection.GeneratedProtocolMessageType('AddDocumentRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDDOCUMENTREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddDocumentRequest)
  })
_sym_db.RegisterMessage(AddDocumentRequest)

AddDocumentResponse = _reflection.GeneratedProtocolMessageType('AddDocumentResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDDOCUMENTRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddDocumentResponse)
  })
_sym_db.RegisterMessage(AddDocumentResponse)

GetAllDocumentNamesRequest = _reflection.GeneratedProtocolMessageType('GetAllDocumentNamesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETALLDOCUMENTNAMESREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllDocumentNamesRequest)
  })
_sym_db.RegisterMessage(GetAllDocumentNamesRequest)

GetAllDocumentNamesResponse = _reflection.GeneratedProtocolMessageType('GetAllDocumentNamesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETALLDOCUMENTNAMESRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllDocumentNamesResponse)
  })
_sym_db.RegisterMessage(GetAllDocumentNamesResponse)

GetDocumentTextRequest = _reflection.GeneratedProtocolMessageType('GetDocumentTextRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDOCUMENTTEXTREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetDocumentTextRequest)
  })
_sym_db.RegisterMessage(GetDocumentTextRequest)

GetDocumentTextResponse = _reflection.GeneratedProtocolMessageType('GetDocumentTextResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDOCUMENTTEXTRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetDocumentTextResponse)
  })
_sym_db.RegisterMessage(GetDocumentTextResponse)

GetLabelIndicesInfoRequest = _reflection.GeneratedProtocolMessageType('GetLabelIndicesInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLABELINDICESINFOREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelIndicesInfoRequest)
  })
_sym_db.RegisterMessage(GetLabelIndicesInfoRequest)

GetLabelIndicesInfoResponse = _reflection.GeneratedProtocolMessageType('GetLabelIndicesInfoResponse', (_message.Message,), {

  'LabelIndexInfo' : _reflection.GeneratedProtocolMessageType('LabelIndexInfo', (_message.Message,), {
    'DESCRIPTOR' : _GETLABELINDICESINFORESPONSE_LABELINDEXINFO,
    '__module__' : 'mtap.api.v1.events_pb2'
    # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelIndicesInfoResponse.LabelIndexInfo)
    })
  ,
  'DESCRIPTOR' : _GETLABELINDICESINFORESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelIndicesInfoResponse)
  })
_sym_db.RegisterMessage(GetLabelIndicesInfoResponse)
_sym_db.RegisterMessage(GetLabelIndicesInfoResponse.LabelIndexInfo)

AddLabelsRequest = _reflection.GeneratedProtocolMessageType('AddLabelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDLABELSREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddLabelsRequest)
  })
_sym_db.RegisterMessage(AddLabelsRequest)

AddLabelsResponse = _reflection.GeneratedProtocolMessageType('AddLabelsResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDLABELSRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddLabelsResponse)
  })
_sym_db.RegisterMessage(AddLabelsResponse)

GetLabelsRequest = _reflection.GeneratedProtocolMessageType('GetLabelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLABELSREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelsRequest)
  })
_sym_db.RegisterMessage(GetLabelsRequest)

GenericLabels = _reflection.GeneratedProtocolMessageType('GenericLabels', (_message.Message,), {
  'DESCRIPTOR' : _GENERICLABELS,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GenericLabels)
  })
_sym_db.RegisterMessage(GenericLabels)

GenericLabel = _reflection.GeneratedProtocolMessageType('GenericLabel', (_message.Message,), {
  'DESCRIPTOR' : _GENERICLABEL,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GenericLabel)
  })
_sym_db.RegisterMessage(GenericLabel)

GetLabelsResponse = _reflection.GeneratedProtocolMessageType('GetLabelsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETLABELSRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelsResponse)
  })
_sym_db.RegisterMessage(GetLabelsResponse)

_EVENTS = DESCRIPTOR.services_by_name['Events']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031edu.umn.nlpie.mtap.api.v1'
  _GETALLMETADATARESPONSE_METADATAENTRY._options = None
  _GETALLMETADATARESPONSE_METADATAENTRY._serialized_options = b'8\001'
  _EVENTS.methods_by_name['GetEventsInstanceId']._options = None
  _EVENTS.methods_by_name['GetEventsInstanceId']._serialized_options = b'\202\323\344\223\002\030\022\026/v1/events/instance_id'
  _EVENTS.methods_by_name['OpenEvent']._options = None
  _EVENTS.methods_by_name['OpenEvent']._serialized_options = b'\202\323\344\223\002\032\"\025/v1/events/{event_id}:\001*'
  _EVENTS.methods_by_name['CloseEvent']._options = None
  _EVENTS.methods_by_name['CloseEvent']._serialized_options = b'\202\323\344\223\002\027*\025/v1/events/{event_id}'
  _EVENTS.methods_by_name['GetAllMetadata']._options = None
  _EVENTS.methods_by_name['GetAllMetadata']._serialized_options = b'\202\323\344\223\002 \022\036/v1/events/{event_id}/metadata'
  _EVENTS.methods_by_name['AddMetadata']._options = None
  _EVENTS.methods_by_name['AddMetadata']._serialized_options = b'\202\323\344\223\002)\"$/v1/events/{event_id}/metadata/{key}:\001*'
  _EVENTS.methods_by_name['AddDocument']._options = None
  _EVENTS.methods_by_name['AddDocument']._serialized_options = b'\202\323\344\223\0024\"//v1/events/{event_id}/documents/{document_name}:\001*'
  _EVENTS.methods_by_name['GetAllDocumentNames']._options = None
  _EVENTS.methods_by_name['GetAllDocumentNames']._serialized_options = b'\202\323\344\223\002!\022\037/v1/events/{event_id}/documents'
  _EVENTS.methods_by_name['GetDocumentText']._options = None
  _EVENTS.methods_by_name['GetDocumentText']._serialized_options = b'\202\323\344\223\0021\022//v1/events/{event_id}/documents/{document_name}'
  _EVENTS.methods_by_name['GetLabelIndicesInfo']._options = None
  _EVENTS.methods_by_name['GetLabelIndicesInfo']._serialized_options = b'\202\323\344\223\0028\0226/v1/events/{event_id}/documents/{document_name}/labels'
  _EVENTS.methods_by_name['AddLabels']._options = None
  _EVENTS.methods_by_name['AddLabels']._serialized_options = b'\202\323\344\223\002H\"C/v1/events/{event_id}/documents/{document_name}/labels/{index_name}:\001*'
  _EVENTS.methods_by_name['GetLabels']._options = None
  _EVENTS.methods_by_name['GetLabels']._serialized_options = b'\202\323\344\223\002E\022C/v1/events/{event_id}/documents/{document_name}/labels/{index_name}'
  _EVENTS.methods_by_name['GetAllBinaryDataNames']._options = None
  _EVENTS.methods_by_name['GetAllBinaryDataNames']._serialized_options = b'\202\323\344\223\002 \022\036/v1/events/{event_id}/binaries'
  _EVENTS.methods_by_name['AddBinaryData']._options = None
  _EVENTS.methods_by_name['AddBinaryData']._serialized_options = b'\202\323\344\223\0026\"1/v1/events/{event_id}/binaries/{binary_data_name}:\001*'
  _EVENTS.methods_by_name['GetBinaryData']._options = None
  _EVENTS.methods_by_name['GetBinaryData']._serialized_options = b'\202\323\344\223\0023\0221/v1/events/{event_id}/binaries/{binary_data_name}'
  _GETEVENTSINSTANCEIDREQUEST._serialized_start=160
  _GETEVENTSINSTANCEIDREQUEST._serialized_end=188
  _GETEVENTSINSTANCEIDRESPONSE._serialized_start=190
  _GETEVENTSINSTANCEIDRESPONSE._serialized_end=240
  _OPENEVENTREQUEST._serialized_start=242
  _OPENEVENTREQUEST._serialized_end=354
  _OPENEVENTRESPONSE._serialized_start=356
  _OPENEVENTRESPONSE._serialized_end=410
  _CLOSEEVENTREQUEST._serialized_start=412
  _CLOSEEVENTREQUEST._serialized_end=467
  _CLOSEEVENTRESPONSE._serialized_start=469
  _CLOSEEVENTRESPONSE._serialized_end=506
  _GETALLMETADATAREQUEST._serialized_start=508
  _GETALLMETADATAREQUEST._serialized_end=549
  _GETALLMETADATARESPONSE._serialized_start=552
  _GETALLMETADATARESPONSE._serialized_end=694
  _GETALLMETADATARESPONSE_METADATAENTRY._serialized_start=647
  _GETALLMETADATARESPONSE_METADATAENTRY._serialized_end=694
  _ADDMETADATAREQUEST._serialized_start=696
  _ADDMETADATAREQUEST._serialized_end=762
  _ADDMETADATARESPONSE._serialized_start=764
  _ADDMETADATARESPONSE._serialized_end=785
  _GETALLBINARYDATANAMESREQUEST._serialized_start=787
  _GETALLBINARYDATANAMESREQUEST._serialized_end=835
  _GETALLBINARYDATANAMESRESPONSE._serialized_start=837
  _GETALLBINARYDATANAMESRESPONSE._serialized_end=895
  _ADDBINARYDATAREQUEST._serialized_start=897
  _ADDBINARYDATAREQUEST._serialized_end=984
  _ADDBINARYDATARESPONSE._serialized_start=986
  _ADDBINARYDATARESPONSE._serialized_end=1009
  _GETBINARYDATAREQUEST._serialized_start=1011
  _GETBINARYDATAREQUEST._serialized_end=1077
  _GETBINARYDATARESPONSE._serialized_start=1079
  _GETBINARYDATARESPONSE._serialized_end=1123
  _ADDDOCUMENTREQUEST._serialized_start=1125
  _ADDDOCUMENTREQUEST._serialized_end=1200
  _ADDDOCUMENTRESPONSE._serialized_start=1202
  _ADDDOCUMENTRESPONSE._serialized_end=1223
  _GETALLDOCUMENTNAMESREQUEST._serialized_start=1225
  _GETALLDOCUMENTNAMESREQUEST._serialized_end=1271
  _GETALLDOCUMENTNAMESRESPONSE._serialized_start=1273
  _GETALLDOCUMENTNAMESRESPONSE._serialized_end=1326
  _GETDOCUMENTTEXTREQUEST._serialized_start=1328
  _GETDOCUMENTTEXTREQUEST._serialized_end=1393
  _GETDOCUMENTTEXTRESPONSE._serialized_start=1395
  _GETDOCUMENTTEXTRESPONSE._serialized_end=1434
  _GETLABELINDICESINFOREQUEST._serialized_start=1436
  _GETLABELINDICESINFOREQUEST._serialized_end=1505
  _GETLABELINDICESINFORESPONSE._serialized_start=1508
  _GETLABELINDICESINFORESPONSE._serialized_end=1802
  _GETLABELINDICESINFORESPONSE_LABELINDEXINFO._serialized_start=1624
  _GETLABELINDICESINFORESPONSE_LABELINDEXINFO._serialized_end=1802
  _GETLABELINDICESINFORESPONSE_LABELINDEXINFO_LABELINDEXTYPE._serialized_start=1748
  _GETLABELINDICESINFORESPONSE_LABELINDEXINFO_LABELINDEXTYPE._serialized_end=1802
  _ADDLABELSREQUEST._serialized_start=1805
  _ADDLABELSREQUEST._serialized_end=2022
  _ADDLABELSRESPONSE._serialized_start=2024
  _ADDLABELSRESPONSE._serialized_end=2043
  _GETLABELSREQUEST._serialized_start=2045
  _GETLABELSREQUEST._serialized_end=2124
  _GENERICLABELS._serialized_start=2126
  _GENERICLABELS._serialized_end=2205
  _GENERICLABEL._serialized_start=2208
  _GENERICLABEL._serialized_end=2371
  _GETLABELSRESPONSE._serialized_start=2374
  _GETLABELSRESPONSE._serialized_end=2504
  _EVENTS._serialized_start=2507
  _EVENTS._serialized_end=4515
# @@protoc_insertion_point(module_scope)
